<?php

  /**
   * @author J. Fager
   * @author Pablo Uriarte
   * @author Mario A. del Riego
   * @author Felipe Zipitria
   * @author Martin Dougiamas
   * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
   * @package moodle multiauth
   *
   * Authentication Plugin: SECIU Authentication
   *
   * Authenticates against SeCIU RADIUS server.
   * Contributed by Felipe Zipitria <fzipi@fing.edu.uy>
   * Contributed by Mario A. del Riego <delriego@fing.edu.uy>
   * Based on RADIUS authenticator by Clive Gould <clive@ce.bromley.ac.uk>
   * CHAP support contributed by Stanislav Tsymbalov http://www.tsymbalov.net/
   *
   * $Id: auth.php 690 2012-09-20 17:32:41Z delriego $
   *
   * 2006-08-31  File created.
   * 2008-03-12  CHAP support added by Stanislav Tsymbalov.
   * 2012-09-20  Support to SECIUS Radius server for Moodle 2.0
   */

if (!defined('MOODLE_INTERNAL')) {
  die('Direct access to this script is forbidden.');    ///  It must be included from a Moodle page
}

if (!defined('AUTH_SECIUTIMEOUT')) {  // timewindow for the Seciu login process, in secs...
  define('AUTH_SECIUTIMEOUT', 300);
}

require_once($CFG->libdir.'/authlib.php');

/**
 * RADIUS authentication plugin.
 */
class auth_plugin_seciu2 extends auth_plugin_base {

  /**
   * Constructor.
   */
  function auth_plugin_seciu2() {
    $this->authtype = 'seciu2';
    $this->config = get_config('auth/seciu2');
    $this->userfields[] = 'field_lock_firstname';
    $this->userfields[] = 'field_lock_lastname';
    $this->userfields[] = 'field_lock_idnumber';
    $this->config->field_updatelocal_firstname = '';
    $this->config->field_updatelocal_lastname = '';
    $this->config->field_updatelocal_country= '';
    $this->config->field_updatelocal_city = '';
    $this->config->field_updatelocal_idnumber = '';
    $this->config->field_updatelocal_institution = '';
    $this->config->field_lock_institution = 'unlockedifempty';
    $this->config->field_lock_idnumber = 'unlockedifempty';
    $this->config->field_lock_city = 'unlockedifempty';
    $this->config->field_lock_firstname = 'unlockedifempty';
    $this->config->field_lock_lastname  = 'unlockedifempty';
  }

  /**
   * Returns true if the username and password work and false if they are
   * wrong or don't exist.
   *
   * @param string $username The username (with system magic quotes)
   * @param string $password The password (with system magic quotes)
   * @return bool Authentication success or failure.
   */
  function user_login ($username, $password) {
    global $CFG, $USER, $SESSION;
	
    /* A. Blanco - 27/3/2011                                                */
    /* first check if user has correct format, otherwise, don't call radius */
    /* user HAS TO BE in the form:
     *                            [0-9]{6,7}@[a-z]+
     */

    $pattern = '/^[0-9]{6,7}@[a-z]+/';

    if (!preg_match($pattern,$username)) {
      // Usuario NO VALIDO
      return false;
    }

    require_once 'Auth/RADIUS.php';
    require_once 'Crypt/CHAP.php';

    // Added by Clive on 7th May for test purposes
    // printf("Username: $username <br/>");
    // printf("Password: $password <br/>");
    // printf("host: $this->config->host <br/>");
    // printf("nasport: $this->config->nasport <br/>");
    // printf("secret: $this->config->secret <br/>");

    // Added by Stanislav Tsymbalov on 12th March 2008 only for test purposes
    //$type = 'PAP';
    //$type = 'CHAP_MD5';
    //$type = 'MSCHAPv1';
    //$type = 'MSCHAPv2';
    $type = $this->config->radiustype;
    if (empty($type)) {
      $type = 'PAP';
    }

    $classname = 'Auth_RADIUS_' . $type;
    $rauth = new $classname(stripslashes($username), stripslashes($password));
    $rauth->addServer($this->config->host, $this->config->nasport, $this->config->secret);

    $rauth->username = $username;

    switch($type) {
    case 'CHAP_MD5':
    case 'MSCHAPv1':
      $classname = $type == 'MSCHAPv1' ? 'Crypt_CHAP_MSv1' : 'Crypt_CHAP_MD5';
      $crpt = new $classname;
      $crpt->password = $password;
      $rauth->challenge = $crpt->challenge;
      $rauth->chapid = $crpt->chapid;
      $rauth->response = $crpt->challengeResponse();
      $rauth->flags = 1;
      // If you must use deprecated and weak LAN-Manager-Responses use this:
      // $rauth->lmResponse = $crpt->lmChallengeResponse();
      // $rauth->flags = 0;
      break;

    case 'MSCHAPv2':
      $crpt = new Crypt_CHAP_MSv2;
      $crpt->username = $username;
      $crpt->password = $password;
      $rauth->challenge = $crpt->authChallenge;
      $rauth->peerChallenge = $crpt->peerChallenge;
      $rauth->chapid = $crpt->chapid;
      $rauth->response = $crpt->challengeResponse();
      break;

    default:
      $rauth->password = $password;
      break;
    }

    if (!$rauth->start()) {
      printf("Radius start: %s<br/>\n", $rauth->getError());
      exit;
    }

    $result = $rauth->send();
    if (PEAR::isError($result)) {
      printf("Radius send failed: %s<br/>\n", $result->getMessage());
      exit;
    } else if ($result === true) {
      // printf("Radius Auth succeeded<br/>\n");
      $key = $username;
      if ($rauth->getAttributes()) {
	$fullname = $rauth->attributes['reply_message'];
	// what if $key is false?
	if (!empty($fullname)) {
	 // deprecated for Moodle 2.9 12/04/2015
	 // fullname is temporary saved in cache table
	 // $textlib = textlib_get_instance();
	  $extfullname = core_text::convert(stripslashes($fullname), 'iso-8859-1', 'utf-8');
	  set_cache_flag('auth/seciu2/fullname', $key, $extfullname, AUTH_SECIUTIMEOUT);
	  debugging('set_cache_flag - key: ' . $key . ' fullname: ' . $extfullname, DEBUG_DEVELOPER);
	}
      }
      $rauth->close();
      return true;
    } else {
      // printf("Radius Auth rejected<br/>\n");
      $rauth->close();
      return false;
    }

    $rauth->close();
    return false;
  }

  function prevent_local_passwords() {
    return true;
  }

  /**
   * Returns true if this authentication plugin is 'internal'.
   *
   * @return bool
   */
  function is_internal() {
    return false;
  }

  /**
   * Returns true if this authentication plugin can change the user's
   * password.
   *
   * @return bool
   */
  function can_change_password() {
    return false;
  }

  /**
   * Prints a form for configuring this authentication plugin.
   *
   * This function is called from admin/auth.php, and outputs a full page with
   * a form for configuring this plugin.
   *
   * @param array $page An object containing all the data for this page.
   */
  function config_form($config, $err, $user_fields) {
    include "config.html";
  }

  /**
   * This function splits a fullname returned from radius auth, filling firstname 
   * and surname user fields.
   *
   * @param string $username Username obtained from seciu2.
   * @return array  $userfields An array containing userfields.
   */
  function split_fullname($username) {
    global $CFG, $USER, $SESSION;

    $userfields = array();
	
    $key = $username;
    $cf = get_cache_flags('auth/seciu2/fullname');
    if (isset($cf[$key]) || $cf[$key] != '') {
      $fullname = $cf[$key];
      $splitted = preg_split("/,/", $fullname);
      $userfields['firstname'] = mb_convert_case($splitted[1], MB_CASE_TITLE, "UTF-8");
      $userfields['lastname'] = mb_convert_case($splitted[0], MB_CASE_TITLE, "UTF-8");
    }

    return $userfields;
  }
  /**
   * reads userinformation from ldap and return it in array()
   *
   * Read user information from external database and returns it as array().
   * Function should return all information available. If you are saving
   * this information to moodle user-table you should honor syncronization flags
   *
   * @param string $username username (with system magic quotes)
   *
   * @return mixed array with no magic quotes or false on error
   */
  function get_userinfo($username) {
    // Wwe have the fullname in session
    global $CFG, $USER, $SESSION;

    $result = array();

    /*
      var $userfields = array(
      'firstname',
      'lastname',
      'email',
      'city',
      'country',
      'lang',
      'description',
      'url',
      'idnumber',
      'institution',
      'department',
      'phone1',
      'phone2',
      'address'
      );

    */

    // Split text received
    $result = $this->split_fullname($username);
    /* sane defaults */
    $result['country'] = 'UY';
    $result['city'] = 'Montevideo';
    if (preg_match('/@/',$username)) { // user has @
      $seciu2info = preg_split('/@/',$username);
      $result['idnumber'] = $seciu2info[0]; // split if it has @
      $result['institution'] = $seciu2info[1]; //todo: must be mapped
    } else {
      $result['idnumber'] = $username;
      $result['institution'] = $this->config->facultad;
            
    }
    return $result;
  }

  /**
   * Update a local user record from an external source.
   * This is a lighter version of the one in moodlelib -- won't do
   * expensive ops such as enrolment.
   *
   * If you don't pass $updatekeys, there is a performance hit and
   * values removed from LDAP won't be removed from moodle.
   *
   * @param string $username username (with system magic quotes)
   */

  function update_user_record($username, $updatekeys = false) {
    global $CFG;

    //just in case check text case
    $username = trim(moodle_strtolower($username));

    // get the current user record
    $user = get_record('user', 'username', $username, 'mnethostid', $CFG->mnet_localhost_id);
    if (empty($user)) { // trouble
      error_log("Cannot update non-existent user: ".stripslashes($username));
      print_error('auth_dbusernotexist','auth','',$username);
      die;
    }
    // Protect the userid from being overwritten
    $userid = $user->id;

    if ($newinfo = $this->get_userinfo($username)) {
      $newinfo = truncate_userinfo($newinfo);

      if (empty($updatekeys)) { // all keys? this does not support removing values
	$updatekeys = array_keys($newinfo);
      }

      foreach ($updatekeys as $key) {
	if (isset($newinfo[$key])) {
	  $value = $newinfo[$key];
	} else {
	  $value = '';
	}
	if (!empty($this->config->{'field_updatelocal_' . $key})) {
	  if ($user->{$key} != $value) { // only update if it's changed
	    set_field('user', $key, addslashes($value), 'id', $userid);
	  }
	}
      }

    } else {
      return false;
    }
    return get_record_select('user', "id = $userid AND deleted = 0");
  }

  /**
   * Processes and stores configuration data for this authentication plugin.
   */
  function process_config($config) {
    // set to defaults if undefined
    if (!isset ($config->host)) {
      $config->host = '127.0.0.1';
    }
    if (!isset ($config->nasport)) {
      $config->nasport = '1812';
    }
    if (!isset($config->radiustype)) {
      $config->radiustype = 'PAP';
    }
    if (!isset ($config->secret)) {
      $config->secret = '';
    }
    if (!isset($config->changepasswordurl)) {
      $config->changepasswordurl = '';
    }
    if (!isset ($config->facultad)) {
      $config->facultad = '';
    }
    if (!isset ($config->field_lock_firstname)) {
      $config->field_lock_firstname = 'unlockedifempty';
    }
    if (!isset ($config->field_lock_lastname)) {
      $config->field_lock_lastname = 'unlockedifempty';
    }
    if (!isset ($config->field_lock_country)) {
      $config->field_lock_country= 'unlockedifempty';
    }
    if (!isset ($config->field_lock_idnumber)) {
      $config->field_lock_idnumber = 'unlockedifempty';
    }

    // save settings

    set_config('host',    $config->host,    'auth/seciu2');
    set_config('nasport', $config->nasport, 'auth/seciu2');
    set_config('secret',  $config->secret,  'auth/seciu2');
    set_config('changepasswordurl', $config->changepasswordurl, 'auth/seciu2');
    set_config('radiustype', $config->radiustype, 'auth/seciu2');
    set_config('facultad', $config->facultad,    'auth/seciu2');
    set_config('field_lock_firstname', $config->field_lock_firstname,    'auth/seciu2');
    set_config('field_lock_lastname', $config->field_lock_lastname,    'auth/seciu2');
    set_config('field_lock_country', $config->field_lock_country,    'auth/seciu2');
    set_config('field_lock_idnumber', $config->field_lock_idnumber,    'auth/seciu2');


    return true;
  }

}

